//	Release Version:	$ ODF 2 $
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.

//========================================================================================
#include "DataCopy.hpp"

#ifndef CONTENT_H
#include "Content.h"
#endif

#ifndef PART_H
#include "Part.h"
#endif

#ifndef PIZZA_H
#include "Pizza.h"			//  CPizza
#endif

// ----- Framework Layer -----
#ifndef FWEVENTU_H
#include "FWEventU.h"		// FW_IsCommandKeyPressed
#endif

#ifndef FWKIND_H			// FW_CKind
#include "FWKind.h"
#endif

//--- OpenDoc ------------------------------------------------------------------
#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>		// kODPropContents
#endif

//========================================================================================
#ifdef FW_BUILD_MAC
#pragma segment DataCopy
#endif

//========================================================================================
FW_DEFINE_AUTO(CPizzaCollection)
FW_DEFINE_AUTO(CPizzaCollectionIterator)
FW_DEFINE_AUTO(CDataCopyContent)

//----------------------------------------------------------------------------------------
CDataCopyContent::CDataCopyContent(Environment* ev, CDataCopyPart* part)
 :	FW_CContent(ev, part),
	fPart(part),
	fNumPizzas(0),
	fPizzaList(NULL)
{
	fPizzaList = FW_NEW(CPizzaCollection, ());
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
CDataCopyContent::~CDataCopyContent()
{
	FW_START_DESTRUCTOR
	this->MyRemoveAllPizzas();
	delete fPizzaList;
}

//----------------------------------------------------------------------------------------
CPizzaCollection*
CDataCopyContent::MyGetPizzaList()
{
	return fPizzaList;
}

//----------------------------------------------------------------------------------------
void
CDataCopyContent::MyIncrement(Environment* ev, FW_CPoint& position)
{
	fNumPizzas++;
	FW_CPoint halfSize( FW_IntToFixed(15), FW_IntToFixed(15) );
	FW_CRect bounds(position - halfSize, position + halfSize);
	CPizza* pizza = new CPizza(bounds);
	fPizzaList->AddLast(pizza);
	fPart->Changed(ev);
	fPart->MyInvalidatePresentation(ev, bounds);
}

//----------------------------------------------------------------------------------------
void
CDataCopyContent::MyAddPizza(Environment* ev, CPizza* pizza)
{
	fNumPizzas++;
	fPizzaList->AddLast(pizza);
	fPart->MyInvalidatePresentation(ev, pizza->GetBounds());
}

//----------------------------------------------------------------------------------------
void
CDataCopyContent::MyRemoveLastPizza(Environment* ev, CPizza* pizza)
{
	fNumPizzas--;
	fPizzaList->RemoveLast();
	fPart->MyInvalidatePresentation(ev, pizza->GetBounds());
}

//----------------------------------------------------------------------------------------
void
CDataCopyContent::MyRemoveAllPizzas()
{
	CPizza* pizza = NULL;
	while ((pizza = (CPizza*)fPizzaList->First()) != NULL) {
		fPizzaList->Remove(pizza);
		delete pizza;
		}
	fNumPizzas = 0;
}

//----------------------------------------------------------------------------------------
void 
CDataCopyContent::ExternalizeKind(Environment* ev,
								ODStorageUnit* storageUnit,
								FW_CKind* kind,
								FW_StorageKinds storageKind,
								FW_CPromise* promise,
								FW_CCloneInfo* cloneInfo)
{
	FW_UNUSED(cloneInfo);
	FW_UNUSED(kind);
	FW_UNUSED(storageKind);
	FW_UNUSED(promise);
	FW_PStorageUnitSink suSink(ev, storageUnit, kODPropContents, kind->GetType(ev));
	FW_CWritableStream stream(suSink);

	// write number
	stream << fNumPizzas;
	
	// write each item
	CPizzaCollectionIterator iter(fPizzaList);
	CPizza* pizza = NULL;
	for (pizza = iter.First(); iter.IsNotComplete(); pizza = iter.Next())
		FW_WRITE_DYNAMIC_OBJECT(stream, pizza, CPizza);

#ifdef FW_BUILD_MAC
	if (::FW_IsCommandKeyPressed()) {
		FW_LOG_MESSAGE("Number of pizzas =");
		Str255 numString;
		::NumToString(fNumPizzas, numString);
		DebugStr(numString);
	}
#endif
}

//----------------------------------------------------------------------------------------
FW_Boolean 
CDataCopyContent::InternalizeKind(Environment* ev,
									ODStorageUnit* storageUnit, 
									FW_CKind* kind,
									FW_StorageKinds storageKind,
									FW_CCloneInfo* cloneInfo)
{
	FW_UNUSED(cloneInfo);
	FW_PStorageUnitSink suSink(ev, storageUnit, kODPropContents, kind->GetType(ev));
	FW_CReadableStream stream(suSink);	

	this->MyRemoveAllPizzas();	// remove current pizzas in case of Paste

	// read number
	stream >> fNumPizzas;
	// read items; make list
	CPizza* pizza = NULL;
	for (long count = 0; count < fNumPizzas; count++) {
		FW_READ_DYNAMIC_OBJECT(stream, &pizza, CPizza);
		fPizzaList->AddLast(pizza);
		}
	if (storageKind != FW_kPartStorage) {
		fPart->MyInvalidatePresentation(ev);
//		fPart->Changed(ev);
		}
	return true;
}
